﻿class MyAjax {
    constructor() {
        // Для исключения конфликтов ссылок this
        const myajax_this = this;
        this.initHtmlForm(myajax_this);
        this.initHtmlFormManually(myajax_this);
    }

    /**
     * Подготовка для отправки данных формы
     * @param myajax_this - текущий объект класса
     */
    initHtmlForm(myajax_this: MyAjax) {
        // Регистрируем обработчик события 
        // отправки данных формы.
        const myForm = document.getElementById("formProduct") as HTMLFormElement;
        myForm?.addEventListener('submit', function (e) {
            // Отключаем обработку события браузером
            e.preventDefault();

            // Очищаем поле отклика сервера.
            myajax_this.replyServer("");

            // Получаем данные формы
            const formData = new FormData(this);

            // Валидация данных на стороне клиента
            const nameProduct = formData.get("nameProduct") as string;
            if (nameProduct.length == 0) {
                alert("Введите название товара.");
            }
            else {
                //myajax_this.formPostPut(formData);
                myajax_this.formPostPutAjax(formData);
            }
        });
    }

    /**
     * Подготовка для отправки данных формы сформированных вручную.
     * @param myajax_this
     */
    initHtmlFormManually(myajax_this: MyAjax) {
        const btnSendFormManually = document.getElementById("btnSendFormManually") as HTMLButtonElement;

        btnSendFormManually.addEventListener('click', function (e) {
            const myForm = document.getElementById("formProduct") as HTMLFormElement;
            // Получаем данные формы
            const formData = new FormData(myForm);

            if (btnSendFormManually) {
                myajax_this.formManually(formData);
            }
        });
    }


    /**
     * Отправка данных формы методами POST PUT
     * @param fd - входные данные формы
     */
    formPostPut(fd: FormData) {
        // Адрес отправки данных формы
        const url = '/html-form';

        // Настройки отправки
        const options = {
            method: 'POST', // "PUT"
            body: fd
        };

        // Асинхронная отправка, основанная на Promise (Обещаниях).
        fetch(url, options)
            .then(response => response.text())
            .then(data => {
                // Обработка ответа сервера
                this.replyServer(data);
            })
            .catch(error => {
                // Обрабатываем ошибку
                console.error("error: " + error);
            });

    }

    /**
     * Отправка данных формы методами POST PUT
     * интерфейсом AJAX
     * @param fd - входные данные формы
     */
    formPostPutAjax(fd: FormData) {
        // Адрес отправки данных формы
        const url = '/html-form';

        // Для исключения конфликтов this внутри объекта XMLHttpRequest.
        const _this = this;

        // Создаем объект AJAX XMLHttpRequest
        var xhr = new XMLHttpRequest();
        xhr.open('POST', url, true);
        // Возврат токена на сервер для проверки на
        // подделку запросов от стороннего сайта(XSRF или CSRF).
        xhr.setRequestHeader('RequestVerificationToken', fd.get('__RequestVerificationToken').toString());
        xhr.onreadystatechange = function () {
            if (xhr.readyState == 4 && xhr.status == 200) {
                // Получаем ответ сервера
                _this.replyServer(xhr.responseText);
            }
        };

        // Отправка данных формы
        xhr.send(fd);
    }


    /**
    * Отправка вручную сформированных данных формы 
    * @param fd
    */
    formManually(fd: FormData) {
        // Алрес отправки данных формы
        const url = '/html-form-manually';

        // Настройки отправки
        // Вариант ручного формирование отправки данных формы
        const options = {
            method: 'POST',
            headers: {
                // Заголовок для исключения фальшивых запросов от сторонних сайтов.
                'RequestVerificationToken': fd.get('__RequestVerificationToken').toString(),
                'Content-Type': 'application/x-www-form-urlencoded'
            },
            // Формирование отправляемой строки.
            body: "nameProduct=" + fd.get("nameProduct") + "&descriptionProduct=" + fd.get("descriptionProduct")
        };

        // Отправка данных, получение отклика сервера, обработка ошибок.
        fetch(url, options)
            .then(response => response.text())
            .then(data => {
                // Обрабатываем ответ сервера
                this.replyServer(data);
            })
            .catch(error => {
                // Обрабатываем ошибку
                console.error("error: " + error);
            });

    }

    /**
     * Отклик сервера
     * @param reply
     */
    replyServer(reply: string) {
        const replyServer = document.getElementById("replyServer");
        if (replyServer) replyServer.innerText = reply;
    }
}
const myAjax = new MyAjax();