var MyAjax = /** @class */ (function () {
    function MyAjax() {
        // Для исключения конфликтов ссылок this
        var myajax_this = this;
        this.initHtmlForm(myajax_this);
        this.initHtmlFormManually(myajax_this);
    }
    /**
     * Подготовка для отправки данных формы
     * @param myajax_this - текущий объект класса
     */
    MyAjax.prototype.initHtmlForm = function (myajax_this) {
        // Регистрируем обработчик события 
        // отправки данных формы.
        var myForm = document.getElementById("formProduct");
        myForm === null || myForm === void 0 ? void 0 : myForm.addEventListener('submit', function (e) {
            // Отключаем обработку события браузером
            e.preventDefault();
            // Очищаем поле отклика сервера.
            myajax_this.replyServer("");
            // Получаем данные формы
            var formData = new FormData(this);
            // Валидация данных на стороне клиента
            var nameProduct = formData.get("nameProduct");
            if (nameProduct.length == 0) {
                alert("Введите название товара.");
            }
            else {
                //myajax_this.formPostPut(formData);
                myajax_this.formPostPutAjax(formData);
            }
        });
    };
    /**
     * Подготовка для отправки данных формы сформированных вручную.
     * @param myajax_this
     */
    MyAjax.prototype.initHtmlFormManually = function (myajax_this) {
        var btnSendFormManually = document.getElementById("btnSendFormManually");
        btnSendFormManually.addEventListener('click', function (e) {
            var myForm = document.getElementById("formProduct");
            // Получаем данные формы
            var formData = new FormData(myForm);
            if (btnSendFormManually) {
                myajax_this.formManually(formData);
            }
        });
    };
    /**
     * Отправка данных формы методами POST PUT
     * @param fd - входные данные формы
     */
    MyAjax.prototype.formPostPut = function (fd) {
        var _this = this;
        // Адрес отправки данных формы
        var url = '/html-form';
        // Настройки отправки
        var options = {
            method: 'POST',
            body: fd
        };
        // Асинхронная отправка, основанная на Promise (Обещаниях).
        fetch(url, options)
            .then(function (response) { return response.text(); })
            .then(function (data) {
            // Обработка ответа сервера
            _this.replyServer(data);
        })
            .catch(function (error) {
            // Обрабатываем ошибку
            console.error("error: " + error);
        });
    };
    /**
     * Отправка данных формы методами POST PUT
     * интерфейсом AJAX
     * @param fd - входные данные формы
     */
    MyAjax.prototype.formPostPutAjax = function (fd) {
        // Адрес отправки данных формы
        var url = '/html-form';
        // Для исключения конфликтов this внутри объекта XMLHttpRequest.
        var _this = this;
        // Создаем объект AJAX XMLHttpRequest
        var xhr = new XMLHttpRequest();
        xhr.open('POST', url, true);
        // Возврат токена на сервер для проверки на
        // подделку запросов от стороннего сайта(XSRF или CSRF).
        xhr.setRequestHeader('RequestVerificationToken', fd.get('__RequestVerificationToken').toString());
        xhr.onreadystatechange = function () {
            if (xhr.readyState == 4 && xhr.status == 200) {
                // Получаем ответ сервера
                _this.replyServer(xhr.responseText);
            }
        };
        // Отправка данных формы
        xhr.send(fd);
    };
    /**
    * Отправка вручную сформированных данных формы
    * @param fd
    */
    MyAjax.prototype.formManually = function (fd) {
        var _this = this;
        // Алрес отправки данных формы
        var url = '/html-form-manually';
        // Настройки отправки
        // Вариант ручного формирование отправки данных формы
        var options = {
            method: 'POST',
            headers: {
                // Заголовок для исключения фальшивых запросов от сторонних сайтов.
                'RequestVerificationToken': fd.get('__RequestVerificationToken').toString(),
                'Content-Type': 'application/x-www-form-urlencoded'
            },
            // Формирование отправляемой строки.
            body: "nameProduct=" + fd.get("nameProduct") + "&descriptionProduct=" + fd.get("descriptionProduct")
        };
        // Отправка данных, получение отклика сервера, обработка ошибок.
        fetch(url, options)
            .then(function (response) { return response.text(); })
            .then(function (data) {
            // Обрабатываем ответ сервера
            _this.replyServer(data);
        })
            .catch(function (error) {
            // Обрабатываем ошибку
            console.error("error: " + error);
        });
    };
    /**
     * Отклик сервера
     * @param reply
     */
    MyAjax.prototype.replyServer = function (reply) {
        var replyServer = document.getElementById("replyServer");
        if (replyServer)
            replyServer.innerText = reply;
    };
    return MyAjax;
}());
var myAjax = new MyAjax();
//# sourceMappingURL=ajaxfetchforms.js.map